@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2014-2016 F5 Networks Inc.
+# Copyright 2014-2017 F5 Networks Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -174,12 +174,13 @@ class ListenerServiceBuilder(object):
         :param bigips: Array of BigIP class instances to update.
         """
         vip = self.service_adapter.get_virtual_name(service)
-        vip["pool"] = name
-        for bigip in bigips:
-            v = bigip.tm.ltm.virtuals.virtual
-            if v.exists(name=vip["name"], partition=vip["partition"]):
-                obj = v.load(name=vip["name"], partition=vip["partition"])
-                obj.modify(**vip)
+        if vip:
+            vip["pool"] = name
+            for bigip in bigips:
+                v = bigip.tm.ltm.virtuals.virtual
+                if v.exists(name=vip["name"], partition=vip["partition"]):
+                    obj = v.load(name=vip["name"], partition=vip["partition"])
+                    obj.modify(**vip)
 
     def update_session_persistence(self, service, bigips):
         """Update session persistence for virtual server.
@@ -450,3 +451,215 @@ class ListenerServiceBuilder(object):
                 LOG.error("Error getting virtual server stats: %s", e.message)
 
         return collected_stats
+
+    def _add_policy(self, vs, policy_name, bigip):
+        vs_name = vs['name']
+        vs_partition = vs['partition']
+        policy_partition = 'Common'
+
+        v = bigip.tm.ltm.virtuals.virtual
+        obj = v.load(name=vs_name,
+                     partition=vs_partition)
+        p = obj.policies_s
+        policies = p.get_collection()
+
+        # see if policy already added to virtual server
+        for policy in policies:
+            if policy.name == policy_name:
+                LOG.debug("L7Policy found. Not adding.")
+                return
+
+        try:
+            # not found -- add policy to virtual server
+            p.policies.create(name=policy_name,
+                              partition=policy_partition)
+        except Exception as exc:
+            # Bug in TMOS 12.1 will return a 404 error, but the request
+            # succeeded. Verify that policy was added, and ignore exception.
+            LOG.debug(exc.message)
+            if not p.policies.exists(name=policy_name,
+                                     partition=policy_partition):
+                # really failed, raise original exception
+                raise
+
+        # success
+        LOG.debug("Added L7 policy {0} for virtual sever {1}".format(
+            policy_name, vs_name))
+
+    def _remove_policy(self, vs, policy_name, bigip):
+        vs_name = vs['name']
+        vs_partition = vs['partition']
+        policy_partition = 'Common'
+
+        v = bigip.tm.ltm.virtuals.virtual
+        obj = v.load(name=vs_name,
+                     partition=vs_partition)
+        p = obj.policies_s
+        policies = p.get_collection()
+
+        # find policy and remove from virtual server
+        for policy in policies:
+            if policy.name == policy_name:
+                l7 = p.policies.load(name=policy_name,
+                                     partition=policy_partition)
+                l7.delete()
+                LOG.debug("Removed L7 policy {0} for virtual sever {1}".
+                          format(policy_name, vs_name))
+
+    def _add_irule(self, vs, irule_name, bigip, rule_partition='Common'):
+        vs_name = vs['name']
+        vs_partition = vs['partition']
+
+        v = bigip.tm.ltm.virtuals.virtual
+        obj = v.load(name=vs_name,
+                     partition=vs_partition)
+        r = obj.rules
+        rules = r.get_collection()
+
+        # see if iRule already added to virtual server
+        for rule in rules:
+            if rule.name == irule_name:
+                LOG.debug("iRule found. Not adding.")
+                return
+
+        try:
+            # not found -- add policy to virtual server
+            r.rules.create(name=irule_name,
+                           partition=rule_partition)
+        except Exception as exc:
+            # Bug in TMOS 12.1 will return a 404 error, but the request
+            # succeeded. Verify that policy was added, and ignore exception.
+            LOG.debug(exc.message)
+            if not r.rule.exists(name=irule_name,
+                                 partition=rule_partition):
+                # really failed, raise original exception
+                raise
+
+        # success
+        LOG.debug("Added iRule {0} for virtual sever {1}".format(
+            irule_name, vs_name))
+
+    def _remove_irule(self, vs, irule_name, bigip, rule_partition='Common'):
+        vs_name = vs['name']
+        vs_partition = vs['partition']
+
+        v = bigip.tm.ltm.virtuals.virtual
+        obj = v.load(name=vs_name,
+                     partition=vs_partition)
+        r = obj.rules_s
+        rules = r.get_collection()
+
+        # find iRule and remove from virtual server
+        for rule in rules:
+            if rule.name == irule_name:
+                irule = r.rules.load(name=irule_name,
+                                     partition=rule_partition)
+                irule.delete()
+                LOG.debug("Removed iRule {0} for virtual sever {1}".
+                          format(irule_name, vs_name))
+
+    def apply_esd(self, svc, esd, bigips):
+        profiles = []
+
+        # get virtual server name
+        update_attrs = self.service_adapter.get_virtual_name(svc)
+
+        # start with server tcp profile
+        if 'lbaas_stcp' in esd:
+            # set serverside tcp profile
+            profiles.append({'name': esd['lbaas_stcp'],
+                             'partition': 'Common',
+                             'context': 'serverside'})
+            # restrict client profile
+            ctcp_context = 'clientside'
+        else:
+            # no serverside profile; use client profile for both
+            ctcp_context = 'all'
+
+        # must define client profile; default to tcp if not in ESD
+        if 'lbaas_ctcp' in esd:
+            ctcp_profile = esd['lbaas_ctcp']
+        else:
+            ctcp_profile = 'tcp'
+        profiles.append({'name':  ctcp_profile,
+                         'partition': 'Common',
+                         'context': ctcp_context})
+
+        # SSL profiles
+        if 'lbaas_cssl_profile' in esd:
+            profiles.append({'name': esd['lbaas_cssl_profile'],
+                             'partition': 'Common',
+                             'context': 'clientside'})
+        if 'lbaas_sssl_profile' in esd:
+            profiles.append({'name': esd['lbaas_sssl_profile'],
+                             'partition': 'Common',
+                             'context': 'serverside'})
+
+        # persistence
+        if 'lbaas_persist' in esd:
+            update_attrs['persist'] = [{'name': esd['lbaas_persist']}]
+        if 'lbaas_fallback_persist' in esd:
+            update_attrs['fallbackPersistence'] = esd['lbaas_fallback_persist']
+
+        if profiles:
+            # always use http and oneconnect
+            profiles.append({'name': 'http',
+                             'partition': 'Common',
+                             'context': 'all'})
+            profiles.append({'name': 'oneconnect',
+                             'partition': 'Common',
+                             'context': 'all'})
+            update_attrs['profiles'] = profiles
+
+        # iRules
+        if 'lbaas_irule' in esd:
+            irules = []
+            for irule in esd['lbaas_irule']:
+                irules.append('/Common/' + irule)
+            update_attrs['rules'] = irules
+
+        # L7 policies
+        if 'lbaas_policy' in esd:
+            policies = []
+            for policy in esd['lbaas_policy']:
+                policies.append({'name': policy, 'partition': 'Common'})
+            update_attrs['policies'] = policies
+
+        # udpate BIG-IPs
+        for bigip in bigips:
+            self.vs_helper.update(bigip, update_attrs)
+
+    def remove_esd(self, svc, esd, bigips):
+        # original service object definition of listener
+        vs = self.service_adapter.get_virtual(svc)
+
+        # add back SSL profile for TLS?
+        tls = self.service_adapter.get_tls(svc)
+        if tls:
+            tls['name'] = vs['name']
+            tls['partition'] = vs['partition']
+
+        # remove iRules
+        if 'lbaas_irule' in esd:
+            vs['rules'] = []
+
+        # remove policies
+        if 'lbaas_policy' in esd:
+            vs['policies'] = []
+
+        # reset persistence to original definition
+        if 'pool' in svc:
+            vip_persist = self.service_adapter.get_session_persistence(svc)
+            vs.update(vip_persist)
+
+        for bigip in bigips:
+            try:
+                # update VS back to original listener definition
+                self.vs_helper.update(bigip, vs)
+
+                # add back SSL profile for TLS
+                if tls:
+                    self.add_ssl_profile(tls, bigip)
+            except Exception as err:
+                LOG.exception("Virtual server update error: %s" % err.message)
+                raise
